/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;

import java.util.ArrayList;
/**
 * Classe servant a representer les polynomes
 * @author Docet Giraudet
 *
 */
public class Polynome implements Comparable<Polynome>{
	private ArrayList<Double> coeff;
	private int degre;
	
	/**
	 * Constructeur qui cree un polynome vide
	 *
	 */
	public Polynome()
	{
		degre=-1;
		coeff=new ArrayList<Double>();
	}
	
	/**
	 * fonction qui détermine si un polynome est nul
	 * @return
	 */
	public boolean PolynomeNul()
	{
		if((coeff.size()==0))
		{
			return true;
		}
		else
			return false;
	}
	
	/**
	 * ecrase si la valeur existe deja
	 * @param c le coeff
	 * @param d degre de la variable
	 */
	public void setCoeff(double c,int d)
	{
		if(d<0)
		{
			System.err.println("Polynome : setCoeff : degre positif"+d);
		}
		else
			{
				if(d>degre)
				{
					for(int i=degre+1;i<d;i++)
					{
						coeff.add(new Double(0));
					}
					coeff.add(c);
					degre=d;
				}
				else
					coeff.set(d,c);
			}
		
	}
	
	//accesseurs
	public double getCoeff(int d)
	{
		return coeff.get(d);
	}
	
	public int getDegre()
	{
		return degre;
	}
	
	/**
	 * fonction qui renvoi la derive d'un polynome
	 * @return
	 */
	public Polynome derive() {
		Polynome aux = new Polynome();
		for (int i=0; i<this.degre; i++)
			aux.setCoeff((i+1)*this.getCoeff(i+1),i);
		aux.degre=this.degre--;
		return aux;
	}
	
	/**
	 * fonction qui calcule la derive en un point d'un polynome
	 * @param x
	 * @return
	 */
	public double nombreDerive(double x) {
		double res=0;
		double puiss=1;
		for (int i=0; i<this.degre; i++) {
			res+=(i+1)*this.getCoeff(i+1)*puiss;
			puiss*=x;
		}
		return res;
	}
	
	/**
	 * fonction qui evalue le polynome en un point
	 * @param x
	 * @return
	 */
	public double evalue(double x) {
		double res=0;
		double puiss=1;
		for (int i=0; i<this.degre+1; i++) {
			res+=this.getCoeff(i)*puiss;
			puiss*=x;
		}
		return res;
	}
	
	/**
	 * fonction qui retourne la racine dominante du polynome
	 * en utilisant la méthode de newton
	 * @return
	 */
	public double RacineDominante() {
		// choix du point de depart
		double ksi=Math.abs(this.getCoeff(0)/this.getCoeff(this.degre));
		for (int i=0; i<=this.degre; i++) {
			double nb=1+Math.abs(this.getCoeff(i)/this.getCoeff(this.degre));
			if (ksi<nb) ksi=nb;
		}
		
		double res=ksi+1;
		
		// methode de Newton
		double old=res;
		do {
			old=res;
			res=res-this.evalue(res)/this.nombreDerive(res);
		} while (Math.abs(old-res)>0.00001);
		return res;
	}
	
	/**
	 * fonction qui permet d'ajouter un polynome a un autre
	 * @param p
	 * @return
	 */
	public Polynome add(Polynome p)
	{
		Polynome aux= new Polynome();
		int max= Math.max(p.degre,this.degre);
		for(int i=0;i<=max;i++)
		{
			if(i>p.degre)
				aux.setCoeff(this.getCoeff(i),i);
			else
				if(i>this.degre)
					aux.setCoeff(p.getCoeff(i),i);
				else
				aux.setCoeff(p.getCoeff(i)+this.getCoeff(i),i);
		}
		return aux;
	}
	
	/**
	 * fonction qui permet de soustraire un polynome a un autre
	 * @param p
	 * @return
	 */
	public Polynome sub(Polynome p)
	{
		Polynome aux= new Polynome();
		for(int i=0;i<=p.degre;i++)
		{
			p.setCoeff(-(p.getCoeff(i)),i);
		}
		aux=this.add(p);
		return aux;
	}
	
	/**
	 * fonction qui permet de multiplier deux polynomes
	 * @param p
	 * @return
	 */
	public Polynome mult(Polynome p)
	{
		Polynome aux = new Polynome();
		Polynome aux1 = new Polynome();
		for(int i=0;i<=this.degre;i++)
		{
			for(int j=0;j<=p.degre;j++)
			{
				if((this.getCoeff(i)!=0)||(p.getCoeff(j)!=0))
				{
					aux1.setCoeff(this.getCoeff(i)*p.getCoeff(j),i+j);
					aux=aux1.add(aux);
					aux1=new Polynome();
				}
			}
		}
		return aux;
	}
	
	/**
	 * fonction qui permet de retourner le polynome sous forme de 
	 * chaine de caractere
	 */
	public String toString()
	{
		String s="";
		if(!this.PolynomeNul())
		{
			for(int i = 0;i<this.coeff.size()-1;i++)
			{
				if((coeff.get(i)!=null))
				{
					if(i!=0)
						s+=coeff.get(i)+"x^"+i+" + ";
					else s+=coeff.get(i)+" + ";
				}
			}
			int i = this.coeff.size()-1;
			if((coeff.get(i)!=null) && (coeff.get(i)!=0))
			{
				if(i!=0)
					s+=coeff.get(i)+"x^"+i;
				else s+=coeff.get(i);
			}
			if(s=="")
				s+=0;
		}
		else
			s = "0.0";
		return s;
	}

	/**
	 * fonction qui permet de comparer deux polynome
	 * @param o
	 * @return
	 * 1 si ils sont differents
	 * 0 sinon
	 */
	public int compareTo(Polynome o) {
		if(this.degre==o.degre)
		{
			for(int i=0;i<this.degre;i++)
			{
				if(o.getCoeff(i)!=this.getCoeff(i))
					return 1;
			}
		return 0;
		}
		return 1;
	}
	
	
	
	
	
	
	
	
	
	
}
